# Copyright 2018-2022 The Wazo Authors  (see the AUTHORS file)
# SPDX-License-Identifier: GPL-3.0-or-later

import logging
import jwt as jwt_module

from requests.exceptions import HTTPError
from wazo_auth import BaseMetadata
from wazo_auth_client import Client as AuthClient

logger = logging.getLogger(__name__)


class ConfigurationException(Exception):
    pass


class MissingNestboxAuth(ConfigurationException):
    pass


class MissingNestboxConfig(ConfigurationException):
    pass


class NestboxJWT(BaseMetadata):

    def __init__(self):
        self._jwt = None

    def load(self, dependencies):
        super().load(dependencies)

        try:
            self._nestbox_config = self._extract_config(dependencies['config'])
        except ConfigurationException as e:
            logger.error(e)
            # This is only a workaround for allowing internal tokens to be
            # generated without nestbox auth. This should be removed in favor
            # of a more visible form of error.
            self._nestbox_config = None

        self._user_service = dependencies['user_service']

    def _extract_config(self, config):
        nestbox_config = config.get('nestbox')
        if not nestbox_config:
            raise MissingNestboxConfig('Missing nestbox section configuration')

        auth_config = nestbox_config.get('auth') or {}
        try:
            config['nestbox']['auth']['username'] = auth_config.pop('service_id')
            config['nestbox']['auth']['password'] = auth_config.pop('service_key')
        except KeyError:
            raise MissingNestboxAuth('Missing nestbox authentication configuration')

        return config['nestbox']

    def get_token_metadata(self, login, args):
        metadata = super().get_token_metadata(login, args)
        metadata['xivo_user_uuid'] = metadata['uuid']  # still here to avoid breaking the API
        metadata['pbx_user_uuid'] = metadata['uuid']

        if self._is_valid(self._jwt):
            metadata['jwt'] = self._jwt
            return metadata

        # This is only a workaround for allowing internal tokens to be
        # generated without nestbox auth. This should be removed in favor
        # of a more visible form of error.
        if not self._nestbox_config:
            logger.debug('Missing nestbox authentication configuration. Skipping jwt token.')
            return metadata

        logger.debug('Getting new subscription authorization')
        client = AuthClient(**self._nestbox_config['auth'])
        try:
            nestbox_token = client.token.new(expiration=10)
        except HTTPError as e:
            logger.error(e)
            return metadata

        try:
            metadata['jwt'] = nestbox_token['metadata']['jwt']
        except KeyError:
            logger.warning("Nestbox token doesn't have an active subscription authorization")
            return metadata

        logger.debug('New subscription authorization fetched')
        self._jwt = metadata['jwt']
        return metadata

    def _is_valid(self, jwt):
        if not jwt:
            return False

        try:
            jwt_module.decode(jwt, options={'verify_signature': False, 'verify_aud': False})
        except jwt_module.ExpiredSignatureError:
            logger.debug('Cached subscription authorization is expired: %s', jwt)
            return False
        except Exception as e:
            logger.error(e)
            return False

        return True
