# Copyright 2019-2022 The Wazo Authors  (see the AUTHORS file)
# SPDX-License-Identifier: GPL-3.0-or-later

import sys
import time
import unittest

from unittest.mock import Mock, sentinel as s, patch

import jwt as jwt_module

from hamcrest import (
    assert_that,
    calling,
    has_entries,
    has_key,
    has_properties,
    not_,
    raises
)


class BaseMetadataMock:

    def load(self, dependencies):
        pass

    def get_token_metadata(self, login, args):
        return {'uuid': s.uuid}


class TestLoad(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        sys.modules['wazo_auth'] = Mock(BaseMetadata=BaseMetadataMock)
        from ..plugin import NestboxJWT, MissingNestboxAuth, MissingNestboxConfig

        cls.plugin = NestboxJWT()
        cls.MissingNestboxAuth = MissingNestboxAuth
        cls.MissingNestboxConfig = MissingNestboxConfig

    def test_extract_config_errors(self):
        samples = [
            ({}, self.MissingNestboxConfig),
            ({'nestbox': {}}, self.MissingNestboxConfig),
            ({'nestbox': None}, self.MissingNestboxConfig),
            ({'nestbox': {'auth': None}}, self.MissingNestboxAuth),
            ({'nestbox': {'auth': {}}}, self.MissingNestboxAuth),
            ({'nestbox': {'auth': None}}, self.MissingNestboxAuth),
        ]

        for config, ExceptionClass in samples:
            assert_that(
                calling(self.plugin._extract_config).with_args(config),
                raises(ExceptionClass),
                config,
            )

    def test_extract_config(self):
        config = {'nestbox': {'auth': {'service_id': s.service_id, 'service_key': s.service_key}}}
        result = self.plugin._extract_config(config)

        assert_that(
            result,
            has_entries(auth=has_entries(username=s.service_id, password=s.service_key))
        )

    def test_load_with_missing_configuration(self):
        invalid_dependencies = {
            'config': {'nestbox': None},
            'user_service': Mock(),
        }

        self.plugin.load(invalid_dependencies)

        assert_that(self.plugin, has_properties(_nestbox_config=None))


class TestGetMetadata(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        sys.modules['wazo_auth'] = Mock(BaseMetadata=BaseMetadataMock)

        from ..plugin import NestboxJWT
        cls.NestboxJWT = NestboxJWT

    @classmethod
    def tearDownClass(cls):
        sys.modules.pop('wazo_auth', None)

    def setUp(self):
        self.auth_client = Mock(return_value=Mock(token=Mock(new=Mock())))
        AuthClientMock = Mock(return_value=self.auth_client)
        patch('wazo_plugind_nestbox_official.auth.plugin.AuthClient', AuthClientMock).start()
        self.dependencies = {
            'config': {
                'nestbox': {
                    'auth': {
                        'service_id': s.service_id,
                        'service_key': s.service_key,
                    }
                }
            },
            'user_service': Mock(list_users=Mock(return_value=[{'uuid': s.uuid}]))

        }
        self.plugin = self.NestboxJWT()
        self.plugin.load(self.dependencies)

    def tearDown(self):
        patch.stopall()

    def test_when_not_initialized(self):
        jwt = self._generate_jwt()
        self.auth_client.token.new.return_value = {'metadata': {'jwt': jwt}}

        metadata = self.plugin.get_token_metadata(s.login, s.args)

        assert_that(metadata, has_entries(jwt=jwt))
        self.auth_client.token.new.assert_called_once()

    def test_when_nestbox_does_not_have_jwt(self):
        self.auth_client.token.new.return_value = {'metadata': {}}

        metadata = self.plugin.get_token_metadata(s.login, s.args)

        assert_that(metadata, not_(has_key('jwt')))
        self.auth_client.token.new.assert_called_once()

    def test_when_jwt_is_expired(self):
        self.plugin._jwt = self._generate_jwt(expired=True)
        jwt = self._generate_jwt()
        self.auth_client.token.new.return_value = {'metadata': {'jwt': jwt}}

        metadata = self.plugin.get_token_metadata(s.login, s.args)

        assert_that(metadata, has_entries(jwt=jwt))
        self.auth_client.token.new.assert_called_once()

    def test_when_jwt_is_not_expired(self):
        jwt = self._generate_jwt(expired=False)
        self.plugin._jwt = jwt

        metadata = self.plugin.get_token_metadata(s.login, s.args)

        assert_that(metadata, has_entries(jwt=jwt))

    def _generate_jwt(self, expired=False):
        now = time.time()
        if not expired:
            exp = now + 10000
        else:
            exp = now - 1

        payload = {
            "iss": "Wazo Inc.",
            "aud": "urn:wazo:customers:00000000-0000-0000-0000-000000000000",
            "sub": "wazo products authorization rules",
            "exp": exp,
            "nbf": now,
            "jti": "00000000-0000-0000-0000-000000000000",
            "authorizations": [],
            "iat": now
        }
        return jwt_module.encode(payload, 'secret', algorithm='HS256')
