# Copyright 2018-2020 The Wazo Authors  (see the AUTHORS file)
# SPDX-License-Identifier: GPL-3.0-or-later

import datetime
import logging
import requests

from wazo_auth_client import Client as AuthClient
from nestbox_confd_client import Client as NestboxConfdClient
from wazo_webhookd.plugins.subscription.service import SubscriptionService
from wazo_webhookd.services.helpers import requests_automatic_hook_retry

logger = logging.getLogger(__name__)

SUBSCRIPTION_ID = {'nestbox-account-subscriptions': 'true'}


class Service:

    def load(self, dependencies):
        self.subscription_service = SubscriptionService(dependencies['config'])

        auth_client = AuthClient(**dependencies['config']['auth'])
        try:
            token = auth_client.token.new(expiration=60)
        except requests.ConnectionError:
            logger.error('nestbox-account plugin not loaded: Unable to connect to wazo-auth')
            return
        except requests.HTTPError:
            logger.error('nestbox-account plugin not loaded: Unable to create token')
            return
        tenant_uuid = token['metadata']['tenant_uuid']

        subscriptions = self.subscription_service.list(search_metadata=SUBSCRIPTION_ID)
        for subscription in subscriptions:
            self.subscription_service.delete(subscription.uuid)

        self.subscription_service.create({
            'name': 'NestBox account subscriptions',
            'service': 'nestbox_account',
            'events': ['user_created', 'user_edited', 'user_deleted'],
            'config': {},
            'metadata_': SUBSCRIPTION_ID,
            'owner_tenant_uuid': tenant_uuid,
        })

        logger.debug('Nestbox plugin initialized')

    @staticmethod
    def run(task, config, subscription, event):
        nestbox_config = config['nestbox']
        nestbox_config['auth']['username'] = nestbox_config['auth'].pop('service_id')
        nestbox_config['auth']['password'] = nestbox_config['auth'].pop('service_key')

        with requests_automatic_hook_retry(task):
            token = AuthClient(**nestbox_config['auth']).token.new('wazo_user', expiration=60)['token']

        data = event.get('data')
        if not data:
            logger.warning('Invalid event data: %s', event)
            return

        name = event.get('name')
        body = {
            'engine_uuid': event['origin_uuid'],
            'engine_tenant_uuid': data['tenant_uuid'],
            'engine_account_uuid': data['uuid'],
            'engine_account_subscription': data['subscription_type'],
        }

        if name == 'user_created':
            body['name'] = 'create'
            body['timestamp'] = data['created_at']
        elif name == 'user_edited':
            body['name'] = 'update'
            body['timestamp'] = datetime.datetime.utcnow().isoformat()
        elif name == 'user_deleted':
            body['name'] = 'delete'
            body['timestamp'] = datetime.datetime.utcnow().isoformat()
        else:
            logger.warning('Invalid event name: %s', name)
            return

        nestbox_confd_client = NestboxConfdClient(token=token, **nestbox_config['confd'])
        with requests_automatic_hook_retry(task):
            return nestbox_confd_client.engines.create_account_event(nestbox_config['instance_uuid'], body)
